<?php

namespace App\Test\Controller;

use App\Entity\Domain;
use App\Repository\DomainRepository;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\KernelBrowser;
use Symfony\Bundle\FrameworkBundle\Test\WebTestCase;

class DomainControllerTest extends WebTestCase
{
    private KernelBrowser $client;
    private DomainRepository $repository;
    private string $path = '/domain/';
    private EntityManagerInterface $manager;

    protected function setUp(): void
    {
        $this->client = static::createClient();
        $this->repository = static::getContainer()->get('doctrine')->getRepository(Domain::class);

        foreach ($this->repository->findAll() as $object) {
            $this->manager->remove($object);
        }
    }

    public function testIndex(): void
    {
        $crawler = $this->client->request('GET', $this->path);

        self::assertResponseStatusCodeSame(200);
        self::assertPageTitleContains('Domain index');

        // Use the $crawler to perform additional assertions e.g.
        // self::assertSame('Some text on the page', $crawler->filter('.p')->first());
    }

    public function testNew(): void
    {
        $originalNumObjectsInRepository = count($this->repository->findAll());

        $this->markTestIncomplete();
        $this->client->request('GET', sprintf('%snew', $this->path));

        self::assertResponseStatusCodeSame(200);

        $this->client->submitForm('Save', [
            'domain[domain]' => 'Testing',
            'domain[name]' => 'Testing',
            'domain[enabled]' => 'Testing',
            'domain[defaultSortPolicy]' => 'Testing',
        ]);

        self::assertResponseRedirects('/domain/');

        self::assertSame($originalNumObjectsInRepository + 1, count($this->repository->findAll()));
    }

    public function testShow(): void
    {
        $this->markTestIncomplete();
        $fixture = new Domain();
        $fixture->setDomain('localhost');
        $fixture->setName('My Title');
        $fixture->setEnabled(true);
        $fixture->setDefaultSortPolicy(Domain::SORT_POLICIES[0]);

        $this->manager->persist($fixture);
        $this->manager->flush();

        $this->client->request('GET', sprintf('%s%s', $this->path, $fixture->getId()));

        self::assertResponseStatusCodeSame(200);
        self::assertPageTitleContains('Domain');

        // Use assertions to check that the properties are properly displayed.
    }

    public function testEdit(): void
    {
        $this->markTestIncomplete();
        $fixture = new Domain();
        $fixture->setDomain('localhost');
        $fixture->setName('My Title');
        $fixture->setEnabled(true);
        $fixture->setDefaultSortPolicy(Domain::SORT_POLICIES[0]);

        $this->manager->persist($fixture);
        $this->manager->flush();

        $this->client->request('GET', sprintf('%s%s/edit', $this->path, $fixture->getId()));

        $this->client->submitForm('Update', [
            'domain[domain]' => 'example.com',
            'domain[name]' => 'Something New',
            'domain[enabled]' => false,
            'domain[defaultSortPolicy]' => Domain::SORT_POLICIES[1],
        ]);

        self::assertResponseRedirects('/domain/');

        $fixture = $this->repository->findAll();

        self::assertSame('example.com', $fixture[0]->getDomain());
        self::assertSame('Something New', $fixture[0]->getName());
        self::assertSame(false, $fixture[0]->getEnabled());
        self::assertSame(Domain::SORT_POLICIES[1], $fixture[0]->getDefaultSortPolicy());
    }

    public function testRemove(): void
    {
        $this->markTestIncomplete();

        $originalNumObjectsInRepository = count($this->repository->findAll());

        $fixture = new Domain();
        $fixture->setDomain('My Title');
        $fixture->setOwnerToken('My Title');
        $fixture->setName('My Title');
        $fixture->setEnabled('My Title');
        $fixture->setDefaultSortPolicy('My Title');
        $fixture->setUpdatedAt('My Title');
        $fixture->setCreatedAt('My Title');

        $this->manager->persist($fixture);
        $this->manager->flush();

        self::assertSame($originalNumObjectsInRepository + 1, count($this->repository->findAll()));

        $this->client->request('GET', sprintf('%s%s', $this->path, $fixture->getId()));
        $this->client->submitForm('Delete');

        self::assertSame($originalNumObjectsInRepository, count($this->repository->findAll()));
        self::assertResponseRedirects('/domain/');
    }
}
